@@ -27,7 +27,12 @@
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libedataserver/e-xml-hash-utils.h>
 #include <libedata-cal/e-cal-backend-cache.h>
+#include <libedata-cal/e-cal-backend-util.h>
+#include <libecal/e-cal-component.h>
+#include <libecal/e-cal-time-util.h>
 #include "e-cal-backend-groupwise.h"
 #include "e-gw-connection.h"
 
@@ -35,11 +40,14 @@
 struct _ECalBackendGroupwisePrivate {
 	/* A mutex to control access to the private structure */
 	GMutex *mutex;
-
 	EGwConnection *cnc;
 	ECalBackendCache *cache;
 	gboolean read_only;
 	char *uri;
+	char *username;
+	char *password;
+	CalMode mode;
+	icaltimezone *default_zone;
 };
 
 static void e_cal_backend_groupwise_dispose (GObject *object);
@@ -48,6 +56,9 @@ static void e_cal_backend_groupwise_finalize (GObject *object);
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
 static ECalBackendClass *parent_class = NULL;
 
+/* Time interval in milliseconds for obtaining changes from server and refresh the cache. */
+#define CACHE_REFRESH_INTERVAL 600000
+
 /* Dispose handler for the file backend */
 static void
 e_cal_backend_groupwise_dispose (GObject *object)
@@ -91,6 +102,16 @@ e_cal_backend_groupwise_finalize (GObject *object)
 		priv->cache = NULL;
 	}
 
+	if (priv->username) {
+		g_free (priv->username);
+		priv->username = NULL;
+	}
+
+	if (priv->password) {
+		g_free (priv->password);
+		priv->password = NULL;
+	}
+
 	g_free (priv);
 	cbgw->priv = NULL;
 
@@ -98,7 +119,6 @@ e_cal_backend_groupwise_finalize (GObject *object)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
-
 /* Calendar backend methods */
 
 /* Is_read_only handler for the file backend */
@@ -174,51 +194,57 @@ convert_uri (const char *gw_uri)
 	return vuri;
 }
 
+/* FIXME: This method is also in the file backend. Can be moved to
+ * ECalComponent. */
+static const char *
+get_rid_string (ECalComponent *comp)
+{
+        ECalComponentRange range;
+        struct icaltimetype tt;
+                                                                                   
+        e_cal_component_get_recurid (comp, &range);
+        if (!range.datetime.value)
+                return "0";
+        tt = *range.datetime.value;
+        e_cal_component_free_range (&range);
+                                                                                   
+        return icaltime_is_valid_time (tt) && !icaltime_is_null_time (tt) ?
+                icaltime_as_ical_string (tt) : "0";
+}
+
 /* Initialy populate the cache from the server */
-//static EGwConnectionStatus
-EGwConnectionStatus
+static EGwConnectionStatus
 populate_cache (ECalBackendGroupwisePrivate *priv)
 {
 	EGwConnectionStatus status;
-	SoupSoapParameter *param;
+        ECalComponent *comp;
 	const char *uid;
 	const char *rid;
-	const char *calobj;
         GSList *list = NULL, *l;
 
         /* get all the objects from the server */
-        status = e_gw_connection_get_items (priv->cnc, &list);
+        status = e_gw_connection_get_items (priv->cnc, NULL, &list);
         if (status != E_GW_CONNECTION_STATUS_OK)
                 return status;
-
-        /* update the cache */
-        g_mutex_lock (priv->mutex);
-
-        for (l = list; l; l = l->next) {
-		param = (SoupSoapParameter *) l->data;
-		uid = soup_soap_parameter_get_first_child_by_name (param, "iCalId");
-		rid = soup_soap_parameter_get_first_child_by_name (param, "recurrance");
-		calobj = soup_soap_parameter_get_string_value (param);
-		e_cal_backend_cache_add_component (priv->cache, uid, rid, calobj);
+        
+        for (l = list; l != NULL; l = g_slist_next(l)) {
+                comp = E_CAL_COMPONENT (l->data);
+		e_cal_component_get_uid (comp, &uid);
+                rid = g_strdup (get_rid_string (comp));
+                /* l contains e-cal-components*/
+                e_cal_component_commit_sequence (comp);
+		e_cal_backend_cache_put_component (priv->cache, uid, rid, 
+                                e_cal_component_get_as_string (comp));
         }
-
-	g_mutex_unlock (priv->mutex);
-
-	return E_GW_CONNECTION_STATUS_OK;        
+        return E_GW_CONNECTION_STATUS_OK;        
                                                                                                                     
 }
 
-static void
+static EGwConnectionStatus 
 update_cache (gpointer *data)
 {
-	EGwConnection *cnc;
-	GList *list;
-	EGwConnectionStatus status;
-
-        cnc = E_GW_CONNECTION (data);
-        /* FIXME: Get all changes from the server since last poll  */
-        /* FIXME: parse the response and update the cache*/
-        /*status =  e_gw_connection_get_items_with_Filter (cnc, &list);*/
+        /* FIXME: to implemented using the getDeltas call */
+	return E_GW_CONNECTION_STATUS_OK;
 }
 
 /* Open handler for the file backend */
@@ -228,10 +254,6 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 {
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
-	GnomeVFSURI *vuri;
-	char *real_uri;
-	EGwConnectionStatus status;
-	ECalBackendSyncStatus result;
 	
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
@@ -239,6 +261,7 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 	g_mutex_lock (priv->mutex);
 
 	/* create the local cache */
+        /* FIXME: if the cache already exists - read it and get deltas. */
 	if (priv->cache)
 		g_object_unref (priv->cache);
 	priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (backend)));
@@ -249,52 +272,18 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
-	/* convert the URI */
-	vuri = convert_uri (e_cal_backend_get_uri (E_CAL_BACKEND (backend)));
-	if (!vuri) {
-		g_mutex_unlock (priv->mutex);
-		return GNOME_Evolution_Calendar_UnsupportedMethod;
-	}
-
-	/* FIXME: login to the server only if we're online */
-	/* create connection to server */
-	real_uri = gnome_vfs_uri_to_string ((const GnomeVFSURI *) vuri,
-					    GNOME_VFS_URI_HIDE_USER_NAME |
-		                            GNOME_VFS_URI_HIDE_PASSWORD);
-	priv->cnc = e_gw_connection_new (real_uri,
-					 username ? username : gnome_vfs_uri_get_user_name (vuri),
-					 password ? password : gnome_vfs_uri_get_password (vuri));
-
-	gnome_vfs_uri_unref (vuri);
-	g_free (real_uri);
-			
-	/* As of now we are assuming that logged in user has write rights to calender */
-	/* we need to read actual rights from server when we implement proxy user access */
 	cbgw->priv->read_only = FALSE;
-	
-	if (E_IS_GW_CONNECTION (priv->cnc)) {
-		/* Populate the cache for the first time.*/
-                /* start a timed polling thread set to 10 minutes*/
-                status = populate_cache (priv);
-                if (status != E_GW_CONNECTION_STATUS_OK) {
-			g_object_unref (priv->cnc);
-			priv->cnc = NULL;
-			g_mutex_unlock (priv->mutex);
-			g_warning (G_STRLOC ": Could not populate the cache");
-
-                        return GNOME_Evolution_Calendar_OtherError;
-                }
-
-                /* FIXME : add a symbolic constant for the time-out */
-		g_mutex_unlock (priv->mutex);
-                g_timeout_add (600000, (GSourceFunc) update_cache, (gpointer) cbgw);
-
-		return GNOME_Evolution_Calendar_Success;
-	}
+	priv->mode = CAL_MODE_LOCAL;
+	priv->username = g_strdup (username);
+	priv->password = g_strdup (password);
+        priv->default_zone = icaltimezone_get_utc_timezone ();        
 		
 	g_mutex_unlock (priv->mutex);
 
-	return GNOME_Evolution_Calendar_AuthenticationFailed;
+	/* FIXME: no need to set it online here when we implement the online/offline stuff correctly */
+	e_cal_backend_set_mode (E_CAL_BACKEND (backend), CAL_MODE_REMOTE);
+
+	return GNOME_Evolution_Calendar_Success;
 }
 
 static ECalBackendSyncStatus
@@ -305,6 +294,8 @@ e_cal_backend_groupwise_remove (ECalBackendSync *backend, EDataCal *cal)
 	
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
+
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 /* is_loaded handler for the file backend */
@@ -317,19 +308,106 @@ e_cal_backend_groupwise_is_loaded (ECalBackend *backend)
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
-	return priv->cnc ? TRUE : FALSE;
+	return priv->cache ? TRUE : FALSE;
 }
 
 /* is_remote handler for the file backend */
 static CalMode
 e_cal_backend_groupwise_get_mode (ECalBackend *backend)
 {
+	ECalBackendGroupwise *cbgw;
+	ECalBackendGroupwisePrivate *priv;
+
+	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
+	priv = cbgw->priv;
+
+	return priv->mode;
 }
 
 /* Set_mode handler for the file backend */
 static void
 e_cal_backend_groupwise_set_mode (ECalBackend *backend, CalMode mode)
 {
+	GnomeVFSURI *vuri;
+	char *real_uri;
+	EGwConnectionStatus status;
+	ECalBackendGroupwise *cbgw;
+	ECalBackendGroupwisePrivate *priv;
+
+	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
+	priv = cbgw->priv;
+
+	if (priv->mode == mode) {
+		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
+					   cal_mode_to_corba (mode));
+		return;
+	}
+
+	g_mutex_lock (priv->mutex);
+
+	switch (mode) {
+	case CAL_MODE_REMOTE :/* go online */
+		/* convert the URI */
+		vuri = convert_uri (e_cal_backend_get_uri (E_CAL_BACKEND (backend)));
+		if (!vuri) {
+			e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SET,
+						   GNOME_Evolution_Calendar_MODE_REMOTE);
+		} else {
+			/* create connection to server */
+			real_uri = gnome_vfs_uri_to_string ((const GnomeVFSURI *) vuri,
+							    GNOME_VFS_URI_HIDE_USER_NAME |
+							    GNOME_VFS_URI_HIDE_PASSWORD);
+			priv->cnc = e_gw_connection_new (
+				real_uri,
+				priv->username ? priv->username : gnome_vfs_uri_get_user_name (vuri),
+				priv->password ? priv->password : gnome_vfs_uri_get_password (vuri));
+
+			gnome_vfs_uri_unref (vuri);
+			g_free (real_uri);
+			
+			/* As of now we are assuming that logged in user has write rights to calender */
+			/* we need to read actual rights from server when we implement proxy user access */
+			cbgw->priv->read_only = FALSE;
+	
+			if (E_IS_GW_CONNECTION (priv->cnc)) {
+				/* Populate the cache for the first time.*/
+				/* start a timed polling thread set to 10 minutes*/
+				status = populate_cache (priv);
+				if (status != E_GW_CONNECTION_STATUS_OK) {
+					g_object_unref (priv->cnc);
+					priv->cnc = NULL;
+					g_warning (G_STRLOC ": Could not populate the cache");
+
+					e_cal_backend_notify_mode (backend,
+								   GNOME_Evolution_Calendar_CalListener_MODE_NOT_SET,
+								   GNOME_Evolution_Calendar_MODE_REMOTE);
+				} else {
+                                        g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) update_cache, (gpointer) cbgw);
+
+					priv->mode = CAL_MODE_REMOTE;
+					e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
+								   GNOME_Evolution_Calendar_MODE_REMOTE);
+				}
+			} else {
+				e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SET,
+							   GNOME_Evolution_Calendar_MODE_REMOTE);
+			}
+		}
+		break;
+	case CAL_MODE_LOCAL : /* go offline */
+		g_object_unref (priv->cnc);
+		priv->cnc = NULL;
+		priv->mode = CAL_MODE_LOCAL;
+
+		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
+					   GNOME_Evolution_Calendar_MODE_LOCAL);
+		break;
+	default :
+		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
+					   cal_mode_to_corba (mode));
+	}
+
+	g_mutex_unlock (priv->mutex);
 }
 
 static ECalBackendSyncStatus
@@ -347,9 +425,6 @@ e_cal_backend_groupwise_get_default_object (ECalBackendSync *backend, EDataCal *
 	case ICAL_VTODO_COMPONENT:
 		e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_TODO);
 		break;
-	case ICAL_VJOURNAL_COMPONENT:
-		e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_JOURNAL);
-		break;
 	default:
 		g_object_unref (comp);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
@@ -361,7 +436,7 @@ e_cal_backend_groupwise_get_default_object (ECalBackendSync *backend, EDataCal *
 	return GNOME_Evolution_Calendar_Success;
 }
 
-/* Get_object_component handler for the file backend */
+/* Get_object_component handler for the groupwise backend */
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_object (ECalBackendSync *backend, EDataCal *cal, const char *uid, const char *rid, char **object)
 {
@@ -386,17 +461,17 @@ e_cal_backend_groupwise_get_object (ECalBackendSync *backend, EDataCal *cal, con
 	return GNOME_Evolution_Calendar_ObjectNotFound;
 }
 
-/* Get_timezone_object handler for the file backend */
+/* Get_timezone_object handler for the groupwise backend */
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzid, char **object)
 {
-	ECalBackendGroupwise *cbfile;
+	ECalBackendGroupwise *cbgw;
         ECalBackendGroupwisePrivate *priv;
         icaltimezone *zone;
         icalcomponent *icalcomp;
 
-        cbfile = E_CAL_BACKEND_GROUPWISE (backend);
-        priv = cbfile->priv;
+        cbgw = E_CAL_BACKEND_GROUPWISE (backend);
+        priv = cbgw->priv;
 
         g_return_val_if_fail (tzid != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
 
@@ -421,20 +496,23 @@ e_cal_backend_groupwise_get_timezone (ECalBackendSync *backend, EDataCal *cal, c
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_set_default_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzid)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
-/* Get_objects_in_range handler for the file backend */
+/* Get_objects_in_range handler for the groupwise backend */
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_object_list (ECalBackendSync *backend, EDataCal *cal, const char *sexp, GList **objects)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
-/* get_query handler for the file backend */
+/* get_query handler for the groupwise backend */
 static void
 e_cal_backend_groupwise_start_query (ECalBackend *backend, EDataCalView *query)
 {
@@ -445,30 +523,62 @@ static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_free_busy (ECalBackendSync *backend, EDataCal *cal, GList *users,
 				time_t start, time_t end, GList **freebusy)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 /* Get_changes handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_changes (ECalBackendSync *backend, EDataCal *cal, const char *change_id,
-			      GList **adds, GList **modifies, GList **deletes)
+				     GList **adds, GList **modifies, GList **deletes)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 /* Discard_alarm handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_discard_alarm (ECalBackendSync *backend, EDataCal *cal, const char *uid, const char *auid)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal, const char *calobj, char **uid)
 {
+	ECalBackendGroupwise *cbgw;
+        ECalBackendGroupwisePrivate *priv;
+	icalcomponent *icalcomp;
+
+	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
+	priv = cbgw->priv;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
+
+	/* check the component for validity */
+	icalcomp = icalparser_parse_string (calobj);
+	if (!icalcomp)
+		return GNOME_Evolution_Calendar_InvalidObject;
+
+	/* check if the object exists */
+	switch (priv->mode) {
+	case CAL_MODE_LOCAL :
+		/* in offline mode, we just update the cache */
+		break;
+	case CAL_MODE_ANY :
+	case CAL_MODE_REMOTE :
+		break;
+	default :
+		break;
+	}
+
+	return GNOME_Evolution_Calendar_InvalidObject;
 }
 
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal, const char *calobj, 
 				CalObjModType mod, char **old_object)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 /* Remove_object handler for the file backend */
@@ -477,19 +587,23 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 				const char *uid, const char *rid,
 				CalObjModType mod, char **object)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 /* Update_objects handler for the file backend. */
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_receive_objects (ECalBackendSync *backend, EDataCal *cal, const char *calobj)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_send_objects (ECalBackendSync *backend, EDataCal *cal, const char *calobj)
 {
+	return GNOME_Evolution_Calendar_OtherError;
 }
 
+
 /* Object initialization function for the file backend */
 static void
 e_cal_backend_groupwise_init (ECalBackendGroupwise *cbgw, ECalBackendGroupwiseClass *class)
