@@ -44,6 +44,7 @@ e_file_cache_set_property (GObject *object, guint property_id, const GValue *val
 	EFileCache *cache;
 	EFileCachePrivate *priv;
 	char *dirname;
+	int result;
 
 	cache = E_FILE_CACHE (object);
 	priv = cache->priv;
@@ -52,7 +53,9 @@ e_file_cache_set_property (GObject *object, guint property_id, const GValue *val
 	case PROP_FILENAME :
 		/* make sure the directory for the cache exists */
 		dirname = g_path_get_dirname ((const char *) g_value_get_string (value));
-		if (e_util_mkdir_hier (dirname, 0700) != 0)
+		result = e_util_mkdir_hier (dirname, 0700);
+		g_free (dirname);
+		if (result != 0)
 			break;
 
 		if (priv->xml_hash)
@@ -229,6 +232,32 @@ e_file_cache_get_object (EFileCache *cache, const char *key)
 	return find_data.found_value;
 }
 
+static void
+add_object_to_list (gpointer key, gpointer value, gpointer user_data)
+{
+	GSList **list = user_data;
+
+	list = g_slist_prepend (list, value);
+}
+
+/**
+ * e_file_cache_get_objects:
+ */
+GSList *
+e_file_cache_get_objects (EFileCache *cache)
+{
+	EFileCachePrivate *priv;
+	GSList *list = NULL;
+
+	g_return_val_if_fail (E_IS_FILE_CACHE (cache), NULL);
+
+	priv = cache->priv;
+
+	e_xmlhash_foreach_key (priv->xml_hash, (EXmlHashFunc) add_object_to_list, &list);
+
+	return list;
+}
+
 /**
  * e_file_cache_add_object:
  */
